// Copyright 2022 Esri.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//   http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

import UIKit
import ArcGIS

class ViewController: UIViewController {
    @IBOutlet weak var mapView: AGSMapView!
    
    private func setupMap() {

        let map = AGSMap(basemapStyle: .arcGISTopographic)

        mapView.map = map

        mapView.setViewpoint(AGSViewpoint(latitude: 34.02700, longitude: -118.80500, scale: 72_000))

    }
    

    private func addFeatureLayer(url: URL) -> AGSFeatureLayer {
        let serviceFeatureTable = AGSServiceFeatureTable(url: url)
        let featureLayer = AGSFeatureLayer(featureTable: serviceFeatureTable)
        mapView.map?.operationalLayers.add(featureLayer)
        return featureLayer
    }

    

    private func addTrailheadsLayer() {
        let featureLayer = addFeatureLayer(url: .trailheads)
        
        let pictureMarkerSymbol = AGSPictureMarkerSymbol(url: .trailheadImage)
        pictureMarkerSymbol.height = 18.0
        pictureMarkerSymbol.width = 18.0
        let simpleRenderer = AGSSimpleRenderer(symbol: pictureMarkerSymbol)
        featureLayer.renderer = simpleRenderer
        
        featureLayer.labelsEnabled = true
        let trailHeadsDefinition = makeLabelDefinition()
        featureLayer.labelDefinitions.addObjects(from: [trailHeadsDefinition])
    }


    private func makeLabelDefinition() -> AGSLabelDefinition {
        
        let trailHeadsTextSymbol = AGSTextSymbol()
        trailHeadsTextSymbol.color = .white
        trailHeadsTextSymbol.size = 20.0
        trailHeadsTextSymbol.haloColor = .red
        trailHeadsTextSymbol.haloWidth = 2.0
        trailHeadsTextSymbol.fontFamily = "Noto Sans"
        trailHeadsTextSymbol.fontStyle = .italic
        trailHeadsTextSymbol.fontWeight = .normal

        // Make an arcade label expression.
        let expression = "$feature.TRL_NAME"
        let arcadeLabelExpression = AGSArcadeLabelExpression(arcadeString: expression)
        let labelDefinition = AGSLabelDefinition(labelExpression: arcadeLabelExpression, textSymbol: trailHeadsTextSymbol)
        labelDefinition.placement = .pointAboveCenter
        
        return labelDefinition
    }

    

    private func addTrailsLayer() {
        // Create a trails feature layer, and add it to mapview
        let featureLayer = addFeatureLayer(url: .trails)
        
        // Create simple line symbols.
        let firstClassSymbol = AGSSimpleLineSymbol(style: .solid, color: .purple, width: 3.0)
        let secondClassSymbol = AGSSimpleLineSymbol(style: .solid, color: .purple, width: 4.0)
        let thirdClassSymbol = AGSSimpleLineSymbol(style: .solid, color: .purple, width: 5.0)
        let fourthClassSymbol = AGSSimpleLineSymbol(style: .solid, color: .purple, width: 6.0)
        let fifthClassSymbol = AGSSimpleLineSymbol(style: .solid, color: .purple, width: 7.0)
        
        // Create 5 class breaks.
        let firstClassBreak = AGSClassBreak(description: "Under 500", label: "0 - 500", minValue: 0.0, maxValue: 500.0, symbol: firstClassSymbol)
        let secondClassBreak = AGSClassBreak(description: "501 to 1000", label: "501 - 1000", minValue: 501.0, maxValue: 1000.0, symbol: secondClassSymbol)
        let thirdClassBreak = AGSClassBreak(description: "1001 to 1500", label: "1001 - 1500", minValue: 1001.0, maxValue: 1500.0, symbol: thirdClassSymbol)
        let fourthClassBreak = AGSClassBreak(description: "1501 to 2000", label: "1501 - 2000", minValue: 1501.0, maxValue: 2000.0, symbol: fourthClassSymbol)
        let fifthClassBreak = AGSClassBreak(description: "2001 to 2300", label: "2001 - 2300", minValue: 2001.0, maxValue: 2300.0, symbol: fifthClassSymbol)
        let elevationBreaks = [firstClassBreak, secondClassBreak, thirdClassBreak, fourthClassBreak, fifthClassBreak]
        
        // Create and assign a class breaks renderer to the feature layer.
        let elevationClassBreaksRenderer = AGSClassBreaksRenderer(fieldName: "ELEV_GAIN", classBreaks: elevationBreaks)
        featureLayer.renderer = elevationClassBreaksRenderer
        
        // Set the layer opacity to semi-transparent.
        featureLayer.opacity = 0.75
    }

    

    private func addNoBikeTrailsLayer() {
        // Create a trails feature layer, and add it to mapview.
        let featureLayer = addFeatureLayer(url: URL(string: "https://services3.arcgis.com/GVgbJbqm8hXASVYi/arcgis/rest/services/Trails/FeatureServer/0")!)
        // Write a definition expression to filter for trails that don't permit the use of bikes.
        featureLayer.definitionExpression = "USE_BIKE = 'No'"
        
        // Create and assign a simple renderer to the feature layer.
        let noBikeTrailSymbol = AGSSimpleLineSymbol(style: .dot, color: .red, width: 2.0)
        let noBikeTrailRenderer = AGSSimpleRenderer(symbol: noBikeTrailSymbol)
        featureLayer.renderer = noBikeTrailRenderer
    }

    private func addBikeOnlyTrailsLayer() {
        // Create a trails feature layer, and add it to mapview.
        let featureLayer = addFeatureLayer(url: URL(string: "https://services3.arcgis.com/GVgbJbqm8hXASVYi/arcgis/rest/services/Trails/FeatureServer/0")!)

        // Write a definition expression to filter for trails that do permit the use of bikes.
        featureLayer.definitionExpression = "USE_BIKE = 'Yes'"
        
        // Create and assign a simple renderer to the feature layer.
        let bikeTrailSymbol = AGSSimpleLineSymbol(style: .dot, color: .blue, width: 2.0)
        let bikeTrailRenderer = AGSSimpleRenderer(symbol: bikeTrailSymbol)
        featureLayer.renderer = bikeTrailRenderer
    }

    private func addOpenSpaceLayer() {
        // Create a parks and open spaces feature layer, and add it to mapview.
        let featureLayer = addFeatureLayer(url: .parksAndOpenSpaces)
        
        // Create fill symbols.
        let purpleFillSymbol = AGSSimpleFillSymbol(style: .solid, color: .purple, outline: .none)
        let greenFillSymbol = AGSSimpleFillSymbol(style: .solid, color: .green, outline: .none)
        let blueFillSymbol = AGSSimpleFillSymbol(style: .solid, color: .blue, outline: .none)
        let redFillSymbol = AGSSimpleFillSymbol(style: .solid, color: .red, outline: .none)
        
        // Create a unique value for natural areas, regional open spaces, local parks & regional recreation parks.
        let naturalAreas = AGSUniqueValue(description: "Natural Areas", label: "Natural Areas", symbol: purpleFillSymbol, values: ["Natural Areas"])
        let regionalOpenSpace = AGSUniqueValue(description: "Regional Open Space", label: "Regional Open Space", symbol: greenFillSymbol, values: ["Regional Open Space"])
        let localPark = AGSUniqueValue(description: "Local Park", label: "Local Park", symbol: blueFillSymbol, values: ["Local Park"])
        let regionalRecreationPark = AGSUniqueValue(description: "Regional Recreation Park", label: "Regional Recreation Park", symbol: redFillSymbol, values: ["Regional Recreation Park"])
        
        // Create and assign a unique value renderer to the feature layer.
        let openSpacesUniqueValueRenderer = AGSUniqueValueRenderer(fieldNames: ["TYPE"], uniqueValues: [naturalAreas, regionalOpenSpace, localPark, regionalRecreationPark], defaultLabel: "Open Spaces", defaultSymbol: .none)
        featureLayer.renderer = openSpacesUniqueValueRenderer
        
        // Set the layer opacity to semi-transparent.
        featureLayer.opacity = 0.2
    }

    

    override func viewDidLoad() {
        super.viewDidLoad()
        setupMap()

        addOpenSpaceLayer()

        addTrailsLayer()

        addNoBikeTrailsLayer()
        addBikeOnlyTrailsLayer()

        addTrailheadsLayer()

    }

}


